/*
 *		This code has been based on the NeHe Base code, found
 *		at nehe.gamedev.net.
 *
 *		It was built to demonstrate the four elements of earth,
 *		water, fire, and wind.
 *
 *		If you've found this code useful, please let me know!
 *
 *		Steve Vermeulen
 *		nexius@shaw.ca
 *
 *		July 25 2002
 *
 */

#include "resource.h"
#include <windows.h>		// Header File For Windows
#include <stdio.h>			// Header File For Standard Input/Output
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library

#include "math.h"

HDC			hDC=NULL;		// Private GDI Device Context
HGLRC		hRC=NULL;		// Permanent Rendering Context
HWND		hWnd=NULL;		// Holds Our Window Handle
HINSTANCE	hInstance;		// Holds The Instance Of The Application

bool	keys[256];			// Array Used For The Keyboard Routine
bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default
bool	viewIs2d = true;	// True if window is under the Orthographic view
							// (used for drawing the text more easily)
int		counter = 0;
int		game_state = 0;

const float PI = 3.14159265f;

GLfloat xtrans;
GLfloat ytrans;
GLfloat ztrans;
GLfloat	xrot;				// X Rotation
GLfloat	yrot;				// Y Rotation
GLfloat	zrot;				// Z Rotation
GLfloat	zoom;				// Depth Into The Screen

GLfloat LightAmbient[4];
GLfloat LightDiffuse[4];
GLfloat LightPosition[] =	{ 0.0f, 0.0f, 2.0f, 1.0f };

GLsizei windowWidth;
GLsizei windowHeight;

GLuint	texture[11];			// Storage For 10 Textures
GLint	num_of_textures;
GLuint	base;

GLUquadricObj *quadratic;

GLfloat fade;

// Flag variables

GLfloat flagWidth;
GLfloat flagHeight;
GLfloat flagUnitLength;		// The width of each quad that makes up
							// the flag
GLfloat flag_radius;		// A temporary variable used when calculating
							// the z values for each vertex on the flag
GLfloat angle1;
GLfloat angle2;
GLfloat angle3;

int		flag_units_per_row;
int		flag_units_per_col;

GLfloat flag_frequency;
GLfloat flag_amplitude;

GLfloat flag_vertices[48][48][7];		// The flag vertices, including the
										// texture coordinates

// Fire variables

#define MAX_FIRE_PARTICLES	300

int active_particles;

typedef struct
{
	float life;
	float fade;

	float r, g, b, a;
	float x, y, z;
	float xspeed, yspeed, zspeed;
	float xg, yg, zg;

} particles;

particles fire_particle[MAX_FIRE_PARTICLES];

// Sphere variables

GLfloat sphere_x;
GLfloat sphere_y;
GLfloat sphere_z;
GLfloat sphere_radius;
GLfloat sphere_move_angle;
GLfloat sphere_rotx;
GLfloat sphere_rotz;
GLfloat sphere_x_velocity;
GLfloat sphere_y_velocity;
GLfloat sphere_z_velocity;
GLfloat sphere_y_acceleration;
GLfloat sphere_angle_acceleration;
GLfloat sphere_angle_velocity;

bool	firstSphereDone = false;
int		sphere_state;

// fountain variables

int		circle_res = 40;

GLfloat fountain_vertices[2][41][3];
GLfloat fountain_side_normals[2][41][3];	// Records 16 normals for each side
GLfloat fountain_height;

// Water variables

int water_num_of_rows = 40;
int water_num_of_cols = 40;

GLfloat water_cut_radius;
GLfloat water_unit_length;
GLfloat water_angle;
GLfloat water_radius;
GLfloat water_turbulence;
GLfloat water_wave_cutoff;
GLfloat water_wave_speed;
GLfloat init_water_level;
GLfloat x_ripple;
GLfloat y_ripple;

GLfloat water_normals[40][40][3];
GLfloat water_vertices[40][40][3];

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc

AUX_RGBImageRec *LoadBMP(char *Filename)				// Loads A Bitmap Image
{
	FILE *File=NULL;									// File Handle

	if (!Filename)										// Make Sure A Filename Was Given
	{
		return NULL;									// If Not Return NULL
	}

	File=fopen(Filename,"r");							// Check To See If The File Exists

	if (File)											// Does The File Exist?
	{
		fclose(File);									// Close The Handle
		return auxDIBImageLoad(Filename);				// Load The Bitmap And Return A Pointer
	}

	return NULL;										// If Load Failed Return NULL
}

int LoadGLTextures()									// Load Bitmaps And Convert To Textures
{
	int Status=FALSE;									// Status Indicator

	num_of_textures = 11;

	AUX_RGBImageRec *TextureImage[11];					// Create Storage Space For The Texture

	memset(TextureImage,0,sizeof(void *)*11);           	// Set The Pointer To NULL

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if ((TextureImage[0]=LoadBMP("Data/textures/chrome.bmp"))&&
		(TextureImage[1]=LoadBMP("Data/textures/caustic.bmp"))&&
		(TextureImage[2]=LoadBMP("Data/textures/inside.bmp"))&&
		(TextureImage[3]=LoadBMP("Data/textures/Font.bmp"))&&
		(TextureImage[4]=LoadBMP("Data/textures/ramp.bmp"))&&
		(TextureImage[5]=LoadBMP("Data/textures/particle.bmp"))&&
		(TextureImage[6]=LoadBMP("Data/textures/Wood.bmp"))&&
		(TextureImage[7]=LoadBMP("Data/textures/canadian_flag.bmp"))&&
		(TextureImage[8]=LoadBMP("Data/textures/metal.bmp"))&&
		(TextureImage[9]=LoadBMP("Data/textures/background.bmp"))&&
		(TextureImage[10]=LoadBMP("Data/textures/ball1.bmp")))
	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(num_of_textures, &texture[0]);					// Create Three Textures

		for (int i=0; i<num_of_textures; i++)
		{
			// Create MipMapped Texture
			glBindTexture(GL_TEXTURE_2D, texture[i]);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[i]->sizeX, TextureImage[i]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[i]->data);
		}
	}

	for (int i=0; i<num_of_textures; i++)
	{
		if (TextureImage[i])								// If Texture Exists
		{
			if (TextureImage[i]->data)						// If Texture Image Exists
			{
				free(TextureImage[i]->data);				// Free The Texture Image Memory
			}

			free(TextureImage[i]);							// Free The Image Structure
		}
	}

	return Status;										// Return The Status
}

GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	windowWidth = width;
	windowHeight = height;

	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Begin with an orthographic view to display the first title screen
	glOrtho(0, 400, 300, 0, 10.0f, 0);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}

GLvoid BuildFont(GLvoid)									// Build Our Font Display List
{
	base=glGenLists(256);									// Creating 256 Display Lists
	glBindTexture(GL_TEXTURE_2D, texture[0]);				// Select Our Font Texture
	for (int i=0; i<256; i++)						// Loop Through All 256 Lists
	{
		float cx=float(i%16)/16.0f;						// X Position Of Current Character
		float cy=float(i/16)/16.0f;						// Y Position Of Current Character

		glNewList(base+i,GL_COMPILE);					// Start Building A List
			glBegin(GL_QUADS);								// Use A Quad For Each Character
				glTexCoord2f(cx,1.0f-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2d(0,16);							// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1.0f-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(16,16);							// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1.0f-cy);			// Texture Coord (Top Right)
				glVertex2i(16,0);							// Vertex Coord (Top Right)
				glTexCoord2f(cx,1.0f-cy);					// Texture Coord (Top Left)
				glVertex2i(0,0);							// Vertex Coord (Top Left)
			glEnd();										// Done Building Our Quad (Character)
			glTranslated(15,0,0);							// Move To The Right Of The Character
		glEndList();										// Done Building The Display List
	}														// Loop Until All 256 Are Built
}

GLvoid KillFont(GLvoid)										// Delete The Font From Memory
{
	glDeleteLists(base,256);								// Delete All 256 Display Lists
}


GLvoid glPrint(GLint x, GLint y, int set, const char *fmt, ...)	// Where The Printing Happens
{
	char		text[256];									// Holds Our String
	va_list		ap;											// Pointer To List Of Arguments

	if (fmt == NULL)										// If There's No Text
		return;												// Do Nothing

	va_start(ap, fmt);										// Parses The String For Variables
	    vsprintf(text, fmt, ap);							// And Converts Symbols To Actual Numbers
	va_end(ap);												// Results Are Stored In Text

	if (set>1)												// Did User Choose An Invalid Character Set?
	{
		set=1;												// If So, Select Set 1 (Italic)
	}
	glLoadIdentity();										// Reset The Modelview Matrix
	glTranslated(x,y,0);									// Position The Text (0,0 - Bottom Left)
	glListBase(base-32+(128*set));							// Choose The Font Set (0 or 1)

	if (set==0)												// If Set 0 Is Being Used Enlarge Font
	{
		glScalef(1.5f,2.0f,1.0f);							// Enlarge Font Width And Height
	}

	glCallLists(strlen(text),GL_UNSIGNED_BYTE, text);		// Write The Text To The Screen

}


void normalize(float v[3])
{
	GLfloat d = (float)sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);

	v[0] /= d;
	v[1] /= d;
	v[2] /= d;
}

void normcrossprod(float v1[3], float v2[3], float out[3])
{
	out[0] = v1[1]*v2[2] - v1[2] * v2[1];
	out[1] = v1[2]*v2[0] - v1[0] * v2[2];
	out[2] = v1[0]*v2[1] - v1[1] * v2[0];

	normalize(out);
}

/*
============
updateWaterNormals()

	calculate the new water normals, as they are constantly changing

============
*/

void updateWaterNormals()
{
	float d1[3], d2[3], norm[3];

	for (int row=0; row<water_num_of_rows; row++)
	{
		for (int col=0; col<water_num_of_cols; col++)
		{
			for (int i=0; i<3; i++)
			{
				d1[i] = water_vertices[row+1][col][i] - water_vertices[row][col+1][i];
				d2[i] =  water_vertices[row+1][col+1][i] - water_vertices[row][col][i];
			}

			normcrossprod(d1, d2, norm);

			water_normals[row][col][0] = norm[0];
			water_normals[row][col][1] = norm[1];
			water_normals[row][col][2] = norm[2];
		}
	}
}

/*
============
change_view()

	If the view 2 dimensional, change it to 3 dimensional and vice versa.

============
*/

void change_view()
{
	if (viewIs2d)
	{
		glViewport(0,0,windowWidth,windowHeight);						// Reset The Current Viewport

		glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
		glLoadIdentity();									// Reset The Projection Matrix

		gluPerspective(45.0f,(GLfloat)windowWidth/(GLfloat)windowHeight,0.1f,100.0f);

		glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
		glLoadIdentity();									// Reset The Modelview Matrix

		viewIs2d = false;
	}
	else
	{
		glViewport(0,0,windowWidth,windowHeight);						// Reset The Current Viewport

		glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
		glLoadIdentity();									// Reset The Projection Matrix

		glOrtho(0, 400, 300, 0, 10.0f, 0);

		glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
		glLoadIdentity();									// Reset The Modelview Matrix

		viewIs2d = true;
	}
}

/*
============
InitObjects()

	Initializes all variables required draw each objects

============
*/

void InitObjects()
{

	fade = 0.0f;

	game_state = 0;

	//----------------------------------------------------------
	// FLAG INITIALIZATION
	//----------------------------------------------------------

	// The size of the flag_vertices array can be determined by
	// dividing the flagWidth and flagHeight by the flagUnitLength

	flagHeight = 1.8f;
	flagWidth = 2.4f;
	flagUnitLength = 0.05f;

	flag_units_per_row = (int)(flagWidth/flagUnitLength);
	flag_units_per_col = (int)(flagHeight/flagUnitLength);

	// The amplitude is the maximum distance away from 0 the 
	// z coordinates can be
	flag_amplitude = 0.24f;
	angle1 = 0;
	angle2 = 0;
	angle3 = 0;
	flag_radius = 0;
	flag_frequency = 4.0f;	// Two complete revolutions

	for (int row=0; row<flag_units_per_col; row++)
	{
		for (int col=0; col<flag_units_per_row; col++)
		{
			// The x, y, and z coordinates for the vertex

			flag_vertices[row][col][0] = flagUnitLength * col;
			flag_vertices[row][col][1] = flagUnitLength * row;
			flag_vertices[row][col][2] = 0;

			// The texture coordinates for the quad.  It starts in the top
			// left and goes around clockwise.
			// We want the texture to map the entire flag, not just the one
			// quad, so each quad has to show only a small portion of the 
			// overall texture.

			flag_vertices[row][col][3] = float(col)/flag_units_per_row;		
			flag_vertices[row][col][4] = float(row)/flag_units_per_col;		
			flag_vertices[row][col][5] = float(col+1)/flag_units_per_row;	
			flag_vertices[row][col][6] = float(row+1)/flag_units_per_col;	
		}
	}

	//----------------------------------------------------------
	// WATER INITIALIZATION
	//----------------------------------------------------------

	float angle;		// The angle used to calculate the fountain
						// coordinates
	float radius[2];	// The inside radius for the fountain
						// and the outside
	float norm[3];		

	fountain_height = 0.6f;

	radius[0] = 1.6f;
	radius[1] = 2.0f;

	for (int c=0; c<2; c++)
	{
		angle = 0;
		for (int i=0; i<circle_res; i++)
		{
			// circle_res-1 so that the last vertex meets the beginning vertex
			angle = i * 2.0f*PI/(float)(circle_res-1);

			fountain_vertices[c][i][0] = radius[c] * (float) cos(angle);
			fountain_vertices[c][i][2] = radius[c] * (float) sin(angle);

			if (c==1)		// make the outside vertices higher
							// so it makes more sense for the 
							// ball to roll around.
			{
				fountain_vertices[1][i][1] = 0.2f;
			}
			else
			{
				fountain_vertices[0][i][1] = 0;
			}
		}
	}

	for (int i=0; i<circle_res; i++)
	{
		// since the fountain is a circle the normal is equal
		// to the vector pointing out from the center
		norm[0] = fountain_vertices[0][i][0];
		norm[1] = 0;
		norm[2] = fountain_vertices[0][i][2];

		normalize(norm);

		// inside wall normals
		fountain_side_normals[0][i][0] = -norm[0];
		fountain_side_normals[0][i][1] = -norm[1];
		fountain_side_normals[0][i][2] = -norm[2];
		
		// outside wall normals
		fountain_side_normals[1][i][0] = norm[0];
		fountain_side_normals[1][i][1] = norm[1];
		fountain_side_normals[1][i][2] = norm[2];
	}

	// Make a quad map for the water.  Y values initialize to zero

	// the length of each quad on the water
	water_unit_length	= 0.1f;
	
	// the current height of the waves
	water_radius		= 0.0f;

	// the distance to stop drawing the waves at.
	// used when initially hit, so that it acts
	// as a pulse moving outward from the ripple
	// point. measured in water units
	water_wave_cutoff	= 0;

	// the radius at which to stop drawing the water
	// this is used to make the water fit in the 
	// circular fountain
	water_cut_radius	= 18.0f;

	// equals how many units you want to cover 180 
	// degrees of wave.  the less units the more
	// turbulent the water looks
	water_turbulence	= 9.0f;

	// measures the speed the waves move at
	// the amount of time it takes for the waves
	// to settle down remains the same
	water_wave_speed	= 7.0f;

	// used to calculate the height of each vertex on
	// the quad map
	water_angle			= 0.0f;

	// the y value the water starts at
	init_water_level	= -0.2f;

	// records the point of collision.
	// measured in water-units
	y_ripple;
	x_ripple;

	// calculates the vertices for the quad-map
	for (int row=0; row<water_num_of_rows; row++)
	{
		for (int col=0; col<water_num_of_cols; col++)
		{
			// subtract half the length to center it
			water_vertices[row][col][0] = water_unit_length * col - water_unit_length * water_num_of_cols/2.0f;
			water_vertices[row][col][1] = init_water_level;
			water_vertices[row][col][2] = water_unit_length * row - water_unit_length * water_num_of_rows/2.0f;

			water_normals[row][col][0] = 0;
			water_normals[row][col][1] = 1.0f;
			water_normals[row][col][2] = 0;
		}
	}


	//----------------------------------------------------------
	// FIRE INITIALISATION
	//----------------------------------------------------------

	for (int i=0;i<MAX_FIRE_PARTICLES;i++)					// Initials All The Textures
	{
		fire_particle[i].life=1.0f;						// Give All The Particles Full Life
		fire_particle[i].fade=float(rand()%100)/1000.0f+0.003f;

		fire_particle[i].x = 0;
		fire_particle[i].y = 0;
		fire_particle[i].z = 0;

		fire_particle[i].r = 1.0f;//float(rand()%60)/600.0f + 0.4f;
		fire_particle[i].g = 0.0f;
		fire_particle[i].b = float(rand()%100)/1000.0f+0.003f;
			
		fire_particle[i].xspeed=float((rand()%60)-30.0f);	// X Axis Speed And Direction
		fire_particle[i].yspeed=float((rand()%40)-15.0f);	// Y Axis Speed And Direction
		fire_particle[i].zspeed=float((rand()%60)-30.0f);	// Z Axis Speed And Direction

		fire_particle[i].xg = 0;
		fire_particle[i].yg = 0.8f;
		fire_particle[i].zg = 0;
	}

	yrot = -110.0f;
}


/*
============
InitGL()

	Initialize all variables required draw the GLScene

============
*/

int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	if (!LoadGLTextures())								// Jump To Texture Loading Routine
	{
		return FALSE;									// If Texture Didn't Load Return FALSE
	}

	BuildFont();

	InitObjects();

	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing

	glBlendFunc(GL_ONE, GL_ONE);

	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);		// Setup The Ambient Light
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		// Setup The Diffuse Light
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	// Position The Light
	glEnable(GL_LIGHT1);								// Enable Light One

	quadratic=gluNewQuadric();							// Create A Pointer To The Quadric Object
	gluQuadricNormals(quadratic, GLU_SMOOTH);			// Create Smooth Normals
	gluQuadricTexture(quadratic, GL_TRUE);				// Create Texture Coords

	return TRUE;										// Initialization Went OK
}

// UpdateFPS()
// Calculates FPS and shows the result in the window caption on regular
// intervals. Needs to be called every frame update.
//
void UpdateFPS()
{
	const int FRAMEINTERVAL = 1000;            // Show FPS every 1000th milliseconds
	static DWORD nFrames = 0;                  // Number of frames since last update
	static DWORD nLastTick = GetTickCount();   // Time of last update
	DWORD nTick = GetTickCount();              // Current time
	if(nTick-nLastTick>=FRAMEINTERVAL)
	{	
		float fFPS = 1000.0f*(GLfloat)nFrames/(GLfloat)(nTick-nLastTick);
		nLastTick = nTick;
		nFrames = 0;
		char szFPS[256];
		sprintf(szFPS, "%.1f FPS", fFPS);
		SetWindowText(hWnd, szFPS);
	}
	nFrames++;
}

/*
============
FadeInAndOut()

	introduces each scene of the demo.
	It fades in the color used to display all the text printed
	after calling this method and fades it out after a certain
	amount of frames.

============
*/

void FadeInAndOut()
{
	if (counter == 0)
	{
		fade = 0;
	}
	else if ((counter > 350) || (keys[VK_RIGHT]))
	{
		counter = -1;
		game_state++;
	}
	else if (counter > 200)
	{
		if (fade > 0.0f)
		{
			fade -= 0.01f;
		}
	}
	else if (counter > 50)
	{
		if (fade < 1.0f)
		{
			fade += 0.01f;
		}
	}

	glColor3f(fade, fade, fade);

	// select the font texture
	glBindTexture(GL_TEXTURE_2D, texture[3]);
}

void drawLogs()
{
	// Log 1

	glTranslatef(-1.5f, 0, 0);
	glRotatef(90.0f, 0, 1.0f, 0);

	gluCylinder(quadratic,0.3f,0.3f,3.0f,32,32);	// Draw Our Cylinder

	// Log 2

	glTranslatef(-0.1f, 0.4f, 0.5f);

	glRotatef(-40.0f, 0, 1.0f, 0);
	glRotatef(20.0f, 1.0f, 0, 0);

	gluCylinder(quadratic,0.3f,0.3f,2.0f,32,32);	// Draw Our Cylinder

	// Log 3

	glTranslatef(0.3f, 0.6f, 0.8f);

	glRotatef(100.0f, 0, 1.0f, 0);
	glRotatef(30.0f, 1.0f, 0, 0);

	gluCylinder(quadratic,0.3f,0.3f,2.0f,32,32);	// Draw Our Cylinder

}

void drawFire()
{
	for (int i=0; i<active_particles; i++)
	{
		float x=fire_particle[i].x;			// Grab Our Particle X Position
		float y=fire_particle[i].y;			// Grab Our Particle Y Position
		float z=fire_particle[i].z;			// Particle Z Pos + Zoom

		// Draw The Particle Using Our RGB Values, Fade The Particle Based On It's Life
		// Also base the amount of green in it on it's life, to make it look like actual
		// fire.
		glColor4f(fire_particle[i].r,1.0f-fire_particle[i].life,fire_particle[i].b,fire_particle[i].life);

		glBegin(GL_TRIANGLE_STRIP);							// Build Quad From A Triangle Strip
			glTexCoord2d(1,1); glVertex3f(x+0.5f,y+0.5f,z); // Top Right
			glTexCoord2d(0,1); glVertex3f(x-0.5f,y+0.5f,z); // Top Left
			glTexCoord2d(1,0); glVertex3f(x+0.5f,y-0.5f,z); // Bottom Right
			glTexCoord2d(0,0); glVertex3f(x-0.5f,y-0.5f,z); // Bottom Left
		glEnd();											// Done Building Triangle Strip

		fire_particle[i].x+=fire_particle[i].xspeed/(2000);// Move On The X Axis By X Speed
		fire_particle[i].y+=fire_particle[i].yspeed/(2000);// Move On The Y Axis By Y Speed
		fire_particle[i].z+=fire_particle[i].zspeed/(2000);// Move On The Z Axis By Z Speed

		fire_particle[i].xspeed+=fire_particle[i].xg;		// Take Pull On X Axis Into Account
		fire_particle[i].yspeed+=fire_particle[i].yg;		// Take Pull On Y Axis Into Account
		fire_particle[i].zspeed+=fire_particle[i].zg;		// Take Pull On Z Axis Into Account
		fire_particle[i].life-=fire_particle[i].fade;		// Reduce Particles Life By 'Fade'

		if (fire_particle[i].life<0.0f)						// If Particle Is Burned Out
		{
			fire_particle[i].life=1.0f;								// Give It New Life
			fire_particle[i].fade=float(rand()%100)/1000.0f+0.003f;	// Random Fade Value
			fire_particle[i].x=0.0f;								// Center On X Axis
			fire_particle[i].y=0.0f;								// Center On Y Axis
			fire_particle[i].z=0.0f;								// Center On Z Axis
			fire_particle[i].xspeed=float((rand()%60)-30.0f);		// X Axis Speed And Direction
			fire_particle[i].yspeed=float((rand()%40)-15.0f);		// Y Axis Speed And Direction
			fire_particle[i].zspeed=float((rand()%60)-30.0f);		// Z Axis Speed And Direction
			fire_particle[i].r=1.0f;								
			fire_particle[i].g=0.0f;								
			fire_particle[i].b=float(rand()%100)/1000.0f+0.003f;	// Random amount of blue for
																	// more variation
		}
	}
}

void DrawFireScene()
{
	if (counter==0)
	{
		zoom = 2.0f;
		ytrans = -1.5f;

		// Initialize the active particles to a low number
		// so that when the camera passes through the fire
		// it's not so slow
		active_particles=10;
	}
	else if (counter < 130)
	{
		// while counter < 130 zoom out

		if (active_particles < MAX_FIRE_PARTICLES)
		{
			active_particles++;
		}

		zoom -= 0.06f;
	}
	else if (counter > 330)
	{
		counter = -1;
		game_state++;
	}
	else if (counter > 300)
	{
		// zoom into the heart of the fire as a transition
		// into the next scene

		if (active_particles > 5)
		{
			active_particles-=5;
		}

		zoom += 0.2f;
		
		ytrans += 0.035f;
	}

	glBindTexture(GL_TEXTURE_2D, texture[6]);
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);

	glLoadIdentity();
	glTranslatef(0, ytrans, zoom);
	glRotatef(-110, 0, 1.0f, 0);
	glRotatef(xrot, 0, 0, 1.0f);
	glColor3f(1.0f, 1.0f, 1.0f);

	drawLogs();

	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glBindTexture(GL_TEXTURE_2D, texture[5]);

	glLoadIdentity();									// Reset The View
	glTranslatef(-0.2f,ytrans+0.6f,zoom);

	drawFire();
}


/*
============
getUnitsFromMapCenter(int row, int col)

	This method is used when drawing the water. It returns the
	units from the center of the quad map the passed row and
	column is.

============
*/

GLfloat getUnitsFromMapCenter(int row, int col)
{
	GLfloat x, y, s;

	x = (float)row - (float)water_num_of_rows/2.0f;
	y = (float)col - (float)water_num_of_cols/2.0f;

	s = (float) sqrt(x*x + y*y);

	return s;
}

/*
============
getUnitsFromRippleCenter(int row, int col)

	This method is used when calculating the wave heights. It returns
	the distance from the center the passed row and column is in quad map
	units.

============
*/

GLfloat getUnitsFromRippleCenter(int row, int col)
{
	GLfloat x, y, s;

	x = (float)row - x_ripple;
	y = (float)col - y_ripple;

	s = (float) sqrt(x*x + y*y);

	return s;
}

void drawWater()
{
	glEnable(GL_BLEND);

	glBegin(GL_QUADS);
		for (int row=0; row<water_num_of_rows-1; row++)
		{
			for (int col=0; col<water_num_of_cols-1; col++)
			{
				// All four points must be within the draw radius
				if ((getUnitsFromMapCenter(row, col    ) < water_cut_radius)&&
					(getUnitsFromMapCenter(row+1, col+1) < water_cut_radius)&&
					(getUnitsFromMapCenter(row  , col+1) < water_cut_radius)&&
					(getUnitsFromMapCenter(row+1, col  ) < water_cut_radius))
				{
					glNormal3f(water_normals[row][col][0],water_normals[row][col][1],water_normals[row][col][2]);

					float xmin, xmax, ymin, ymax;

					xmax = (float)(row+1)/(float)(water_num_of_rows);
					xmin = (float)(row)/(float)(water_num_of_rows);
					ymax = (float)(col+1)/(float)(water_num_of_cols);
					ymin = (float)(col)/(float)(water_num_of_cols);

					glTexCoord2f(ymin, xmin);
					glVertex3f(water_vertices[row][col][0], water_vertices[row][col][1], water_vertices[row][col][2]);
					
					glTexCoord2f(ymin, xmax);
					glVertex3f(water_vertices[row+1][col][0], water_vertices[row+1][col][1], water_vertices[row+1][col][2]);
					
					glTexCoord2f(ymax, xmax);
					glVertex3f(water_vertices[row+1][col+1][0], water_vertices[row+1][col+1][1], water_vertices[row+1][col+1][2]);
					
					glTexCoord2f(ymax, xmin);
					glVertex3f(water_vertices[row][col+1][0], water_vertices[row][col+1][1], water_vertices[row][col+1][2]);
				}
			}
		}
	glEnd();

	if (water_radius > 0)
	{
		updateWaterNormals();	// the normals on the water are constantly changing,
								// so we have to update them. 

		for (int row=0; row<water_num_of_rows; row++)
		{
			for (int col=0; col<water_num_of_cols; col++)
			{
				float s = getUnitsFromRippleCenter(row, col);

				// don't draw the quads that haven't been reached by the wave yet
				if (s < water_wave_cutoff)
				{
					/*  The height of the water is based on the sine function 
					 *	The angle used when calculating the height through sine is
					 *	based on the distance away from the ripple center.
					 *
					 *	water_turbulence is the amount of units in 180 degrees of
					 *  sine wave, so multiplying their ratio by the units from
					 *  the ripple center will get the desired angle.
					 *
					 *	I add water_angle as an offset... creates the motion.
					 */

					float temp_angle = s * 180.0f/water_turbulence + water_angle;

					water_vertices[row][col][1] = init_water_level + water_radius * (float) sin(temp_angle * PI/180.0f);
				}
			}
		}

		water_wave_cutoff += water_turbulence * water_wave_speed/180.0f;

		if (water_radius - 0.0004f <= 0)
		{
			water_wave_cutoff = 0.0f;
			water_angle = 0.0f;
			water_radius = 0;
		}
		else
		{
			water_radius -= 0.0004f;
		}

		water_angle-=water_wave_speed;

		if (water_angle > 360.0f)
		{
			water_angle -= 360.0f;
		}
	}

	glDisable(GL_BLEND);
}

void drawFountain()
{
	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glBegin(GL_QUADS);			// Start Drawing Quads
		
		/*
			Draw fountain border top
		*/

		for (int i=0; i<circle_res-1; i++)
		{
			glNormal3f(0, 1.0f, 0);

			glTexCoord2f(0, 0);
			glVertex3f(fountain_vertices[0][i][0], fountain_vertices[0][i][1], fountain_vertices[0][i][2]);

			glTexCoord2f(0, 0.4f);
			glVertex3f(fountain_vertices[1][i][0], fountain_vertices[1][i][1], fountain_vertices[1][i][2]);

			glTexCoord2f(0.4f, 0.4f);
			glVertex3f(fountain_vertices[1][i+1][0], fountain_vertices[1][i+1][1], fountain_vertices[1][i+1][2]);

			glTexCoord2f(0.4f, 0);
			glVertex3f(fountain_vertices[0][i+1][0], fountain_vertices[0][i+1][1], fountain_vertices[0][i+1][2]);
		}
		
		/*
			Draw fountain outside wall
		*/

		for (i=0; i<circle_res-1; i++)
		{
			glNormal3f(fountain_side_normals[1][i][0],fountain_side_normals[1][i][1],fountain_side_normals[1][i][2]);
			
			glTexCoord2f(0, 0.3f);
			glVertex3f(fountain_vertices[1][i][0], fountain_vertices[1][i][1], fountain_vertices[1][i][2]);
			
			glTexCoord2f(0.3f, 0.3f);
			glVertex3f(fountain_vertices[1][i+1][0], fountain_vertices[1][i+1][1], fountain_vertices[1][i+1][2]);
			
			glTexCoord2f(0.3f, 0);
			glVertex3f(fountain_vertices[1][i+1][0], fountain_vertices[1][i+1][1] - fountain_height, fountain_vertices[1][i+1][2]);
			
			glTexCoord2f(0, 0);
			glVertex3f(fountain_vertices[1][i][0], fountain_vertices[1][i][1] -fountain_height, fountain_vertices[1][i][2]);
		}

	glEnd();

	glBindTexture(GL_TEXTURE_2D, texture[2]);

	glBegin(GL_QUADS);

		/*
			Draw fountain inside wall
		*/

		for (i=0; i<circle_res-1; i++)
		{
			glNormal3f(fountain_side_normals[0][i][0],fountain_side_normals[0][i][1],fountain_side_normals[0][i][2]);
			
			glTexCoord2f(0, 0.3f);
			glVertex3f(fountain_vertices[0][i][0], fountain_vertices[0][i][1], fountain_vertices[0][i][2]);
			
			glTexCoord2f(0.3f, 0.3f);
			glVertex3f(fountain_vertices[0][i+1][0], fountain_vertices[0][i+1][1], fountain_vertices[0][i+1][2]);
			
			glTexCoord2f(0.3f, 0);
			glVertex3f(fountain_vertices[0][i+1][0], fountain_vertices[0][i+1][1] - fountain_height, fountain_vertices[0][i+1][2]);
			
			glTexCoord2f(0, 0);
			glVertex3f(fountain_vertices[0][i][0], fountain_vertices[0][i][1] -fountain_height, fountain_vertices[0][i][2]);
		}
	glEnd();

	glBegin(GL_POLYGON);
		/*
			Draw fountain floor
		*/
		glNormal3f(0, 1.0f, 0);

		float angle;
		for (i=0; i<circle_res; i++)
		{
			angle = i * 2.0f * PI/circle_res;
			glTexCoord2f((1.0f + (float)cos(angle))/2.0f, (1.0f + (float)sin(angle))/2.0f);
			glVertex3f(fountain_vertices[0][i][0], -fountain_height, fountain_vertices[0][i][2]);
		}

	glEnd();
	
	glEnable(GL_TEXTURE_2D);
}

/*
============
drawSphere()

	This method draws and performs actions on the moving sphere.

	Case 0 consists of the ball falling down and hitting the ramp
	Case 1 consists of the ball moving around the fountain and
		hitting the ramp again
	Case 2 consists of the ball flying into the air and landing in
		the water
	Case 3 consists of the ball falling down through the water
	Case 4 is called when the first ball hits the floor and changes
		it's radius and other properties to prepare for the next 
		drop
	Case 5 is called when the second ball hits the floor
============
*/

void drawSphere()
{
	switch(sphere_state)
	{
		case 0:			// Sphere falls from above
		{
			sphere_x = sphere_x + sphere_x_velocity;
			sphere_y = sphere_y + sphere_y_velocity;
			sphere_z = sphere_z + sphere_z_velocity;
			
			glTranslatef(sphere_x, sphere_y, sphere_z);

			glRotatef(sphere_rotx, 1.0f, 0, 0);
			glRotatef(sphere_rotz, 0, 1.0f, 0);

			gluSphere(quadratic, sphere_radius, 10, 10);

			sphere_rotx+=5.0f;
			sphere_rotz+=5.0f;

			if (sphere_y < 0.22f)
			{
				// play a looping rolling noise while moving around the 
				// circle
				PlaySound("Data/sounds/roll1.wav", 0, SND_ASYNC | SND_LOOP);

				sphere_state = 1;
			}

			break;
		}
		case 1:		// Sphere moves around circle
		{
			sphere_x = 1.8f * (float) cos(sphere_move_angle * PI/180.0f);
			sphere_z = 1.8f * (float) sin(sphere_move_angle * PI/180.0f);

			glTranslatef(sphere_x, sphere_y, sphere_z);

			glRotatef(sphere_rotx, 1.0f, 0, 0);
			glRotatef(sphere_rotz, 0, 0, 1.0f);

			gluSphere(quadratic, sphere_radius, 10, 10);

			sphere_move_angle+=sphere_angle_velocity;
			sphere_angle_velocity += sphere_angle_acceleration;

			// the following correctly rotates the sphere depending
			// on where it is on the circle
			// if it is at the very front of the circle, move angle is
			// 90 degrees and cos is zero, so it only rotates around the
			// z plane
			sphere_rotx += 4.0f * (float) cos(sphere_move_angle * PI/180.0f);
			sphere_rotz += 4.0f * (float) sin(sphere_move_angle * PI/180.0f);

			// absolute value because the second drop has negative angle velocity
			
			if ((float) fabs(sphere_move_angle) > 355.0f)
				// sphere hits ramp for second time
			{
				sphere_y_velocity =  0.04f;
				sphere_x_velocity = -0.02f;

				if (firstSphereDone)
				{
					sphere_z_velocity = -0.01f;
				}
				else
				{
					sphere_z_velocity =  0.01f;
				}

				PlaySound("Data/sounds/hit1.wav", 0, SND_ASYNC);
				sphere_state = 2;
			}

			break;
		}
		case 2:		// Sphere flies off ramp towards water
		{
			sphere_x = sphere_x + sphere_x_velocity;
			sphere_y = sphere_y + sphere_y_velocity;
			sphere_z = sphere_z + sphere_z_velocity;
			
			glTranslatef(sphere_x, sphere_y, sphere_z);

			sphere_y_velocity -= 0.0008f;

			glRotatef(sphere_rotx, 1.0f, 0, 0);
			glRotatef(sphere_rotz, 0, 1.0f, 0);

			gluSphere(quadratic, sphere_radius, 10, 10);

			sphere_rotx+=5.0f;
			sphere_rotz+=5.0f;

			if (sphere_y < init_water_level + sphere_radius)
			{
				if (firstSphereDone)
				{
					PlaySound("Data/sounds/water2.wav", 0, SND_ASYNC);
					// The height of the water waves
					water_radius = 0.3f;
				}
				else
				{
					PlaySound("Data/sounds/water1.wav", 0, SND_ASYNC);
					// The height of the water waves
					water_radius = 0.1f;
				}
					// Position the ripple origin
				x_ripple = (float)water_num_of_cols/2.0f + sphere_z / water_unit_length;
				y_ripple = (float)water_num_of_rows/2.0f + sphere_x / water_unit_length;
				

				sphere_y_velocity = -0.03f;
				sphere_state = 3;
			}

			break;
		}
		case 3:
		{
			sphere_x = sphere_x + sphere_x_velocity;
			sphere_y = sphere_y + sphere_y_velocity;
			sphere_z = sphere_z + sphere_z_velocity;

			glTranslatef(sphere_x, sphere_y, sphere_z);
			
			gluSphere(quadratic, sphere_radius, 10, 10);

			// if it hits the floor
			if (sphere_y < sphere_radius -0.5f)
			{
				sphere_y_velocity=0;

				if (!firstSphereDone)
				{
					sphere_state = 4;
				}
				else
				{
					sphere_state = 5;
				}
			}

			break;
		}
		case 4:		// set up variables for next drop
		{
			firstSphereDone = true;

			sphere_x = 1.772653955f;
			sphere_y = 3.0f;
			sphere_z = -0.31256672f;

			sphere_angle_velocity = -1.1f;
			sphere_angle_acceleration = 0;
			sphere_move_angle = -10.0f;

			sphere_y_velocity = -0.04f;
			sphere_z_velocity = 0;
			sphere_x_velocity = 0;

			sphere_radius = 0.14f;
			sphere_state = -1;

			break;
		}
		case 5:		// just draw it where it is
		{
			glTranslatef(sphere_x, sphere_y, sphere_z);
			
			gluSphere(quadratic, sphere_radius, 10, 10);

			break;
		}
	}
}

void drawRamp()
{
	glBindTexture(GL_TEXTURE_2D, texture[4]);

	glBegin(GL_TRIANGLES);
		glNormal3f(1.0f, 0, 0);

		glVertex3f(1.9f, 0.1f, -0.3f);
		glVertex3f(1.9f, 0.4f, 0);
		glVertex3f(1.9f, 0.1f, 0.3f);

		glNormal3f(-0.685994f, 0.514496f, -0.514496f);

		glVertex3f(1.9f, 0.1f, -0.3f);
		glVertex3f(1.65f, 0, 0);
		glVertex3f(1.9f, 0.4f, 0);

		glNormal3f(-0.685994f, 0.514496f, 0.514496f);

		glVertex3f(1.9f, 0.1f, 0.3f);
		glVertex3f(1.65f, 0, 0);
		glVertex3f(1.9f, 0.4f, 0);

	glEnd();
}

void DrawWaterScene()
{
	if (counter == 0)
	{
		LightAmbient[0] = 0;
		LightAmbient[1] = 0;
		LightAmbient[2] = 0;
		LightDiffuse[0] = 0;
		LightDiffuse[1] = 0;
		LightDiffuse[2] = 0;

		glEnable(GL_DEPTH_TEST);
		glEnable(GL_LIGHTING);

		sphere_x = 1.7989f;
		sphere_y = 2.0f;
		sphere_z = 0.062819f;

		sphere_angle_velocity = 1.4f;
		sphere_angle_acceleration = 0;

		// begin a little past the ramp
		sphere_move_angle = 2.0f;

		sphere_y_velocity = -0.04f;
		sphere_z_velocity = 0;
		sphere_x_velocity = 0;

		sphere_radius = 0.08f;

		// do nothing until the scene is fully lit
		sphere_state = -1;

		xrot = 90.0f;
		zoom = -5.0f;
	}

	if (counter < 300)		// fade in the scene
	{
		if (LightAmbient[0] < 0.5f)
		{
			LightAmbient[0] += 0.0025f;
			LightAmbient[1] += 0.0025f;
			LightAmbient[2] += 0.0025f;

			glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient );
			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}

		if (LightDiffuse[0] < 1.0f)
		{
			LightDiffuse[0] += 0.005f; 
			LightDiffuse[1] += 0.005f; 
			LightDiffuse[2] += 0.005f; 

			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}
	}
	else if (counter > 1800)	// end scene
	{
		counter = -1;
		game_state++;
	}
	else if (counter > 1600)	// fade out scene
	{
		if (LightAmbient[0] > 0)
		{
			LightAmbient[0] -= 0.0025f;
			LightAmbient[1] -= 0.0025f;
			LightAmbient[2] -= 0.0025f;

			glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient );
			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}

		if (LightDiffuse[0] > 0)
		{
			LightDiffuse[0] -= 0.005f; 
			LightDiffuse[1] -= 0.005f; 
			LightDiffuse[2] -= 0.005f; 

			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}
	}


	if (counter == 820)		// start the sphere from it's beginning
							// again, with the new radius
	{
		sphere_state=0;
	}


	if (xrot > 30)			// while fading in, rotate down until almost
							// level with the fountain
	{
		if (xrot <= 30.4)
		{
			sphere_state = 0;	// drop the sphere
		}

		xrot -= 0.4f;
	}

	glBindTexture(GL_TEXTURE_2D, texture[10]);

	if (firstSphereDone)		// if the first sphere has landed
								// in the water, draw a second sphere
								// in it's place and move the first
								// sphere back to the beginning with
								// a new radius
	{
		glTranslatef(-0.529669f, -0.428798f, 0.973668f);
		
		gluSphere(quadratic, 0.08f, 10, 10);	// draw second sphere

		glTranslatef(0.529669f, 0.428798f, -0.973668f);
	}

	drawSphere();				// draw first sphere

	glLoadIdentity();									// Reset The View
	glTranslatef(0.0f,0.0f,zoom);

	glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(yrot,0.0f,1.0f,0.0f);
	glRotatef(zrot,0.0f,0.0f,1.0f);

	drawFountain();
	
	glBindTexture(GL_TEXTURE_2D, texture[1]);

	drawWater();

	drawRamp();

	yrot+=0.5f;				// rotate around the origin throughout the scene
}

/*
============
DrawFlag()

	This method draws the flag, and calculates the new z-coordinates
	for each vertex on the flag.

============
*/

void DrawFlag()
{
	glBegin(GL_QUADS);

		for (int row=0; row<flag_units_per_col-1; row++)
		{
			for (int col=0; col<flag_units_per_row-1; col++)
			{
				// The normal is constantly changing so we have to
				// update it

				GLfloat d1[3], d2[3], norm[3];

				for (int i=0; i<3; i++)
				{
					d1[i] = flag_vertices[row][col][i] - flag_vertices[row][col+1][i];
					d2[i] = flag_vertices[row][col+1][i] - flag_vertices[row+1][col][i];
				}

				normcrossprod(d1, d2, norm);

				glNormal3f(norm[0], norm[1], norm[2]);

				glTexCoord2f(flag_vertices[row][col][3], flag_vertices[row][col][4]);
				glVertex3f(flag_vertices[row][col][0], flag_vertices[row][col][1], flag_vertices[row][col][2]);

				glTexCoord2f(flag_vertices[row][col][5], flag_vertices[row][col][4]);
				glVertex3f(flag_vertices[row][col+1][0], flag_vertices[row][col+1][1], flag_vertices[row][col+1][2]);

				glTexCoord2f(flag_vertices[row][col][5], flag_vertices[row][col][6]);
				glVertex3f(flag_vertices[row+1][col+1][0], flag_vertices[row+1][col+1][1], flag_vertices[row+1][col+1][2]);

				glTexCoord2f(flag_vertices[row][col][3], flag_vertices[row][col][6]);
				glVertex3f(flag_vertices[row+1][col][0], flag_vertices[row+1][col][1], flag_vertices[row+1][col][2]);
			}
		}
	glEnd();

	// The rest of this method calculates the new z-values for each vertex
	// on the flag.  In order to make the flag look as realistic as possible,
	// I tried to make it move more randomly than just a constant sine function.
	// To do this, I included three angles.  One to constantly change the 
	// frequency (angle3, which determines how many revolutions to make before 
	// reaching the length of the flag (2*PI being one revolution)), one to 
	// constantly change the radius (angle2, which determines the maximum amount
	// the z-value can be) and one to calculate the new z-value based on the 
	// radius (angle1).

	for (int col=0; col<flag_units_per_row; col++)
	{
		angle1=flag_frequency * PI * (float)col/(float)flag_units_per_row;

		flag_radius = (flag_amplitude) * (float) sin(angle2*PI/180.0f);

		for (int row=0; row<flag_units_per_col; row++)
		{
			flag_vertices[row][col][2] =  (flag_radius) * (float) sin(angle1);
		}

		angle3+=0.05f;
		if (angle3 >= 360.0f)
		{
			angle3 = angle3 - 360.0f;
		}

		flag_frequency = 1.0f * (float) sin(angle3*PI/180.0f) + 3.5f;
	}

	angle2+=5.0f;
	if (angle2 >= 360.0f)
	{
		angle2 = angle2 - 360.0f;
	}
}

/*
============
DrawWindScene()

	This method draws the flag, pole, and background that make up
	the wind scene

============
*/

void DrawWindScene()
{
	if (counter == 0)
	{
		PlaySound("data/sounds/flag_wind1.wav", NULL, SND_ASYNC | SND_LOOP);

		// Change to allow for 3d objects
		change_view();

		// Initialize the lights to zero
		LightAmbient[0] = 0;
		LightAmbient[1] = 0;
		LightAmbient[2] = 0;
		LightDiffuse[0] = 0;
		LightDiffuse[1] = 0;
		LightDiffuse[2] = 0;

		glEnable(GL_LIGHTING);

		glColor3f(1.0f, 1.0f, 1.0f);
	}

	// Fade in scene
	if (counter < 200)
	{
		if (LightAmbient[0] < 0.5f)
		{
			LightAmbient[0] += 0.0025f;
			LightAmbient[1] += 0.0025f;
			LightAmbient[2] += 0.0025f;

			glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient );
			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}

		if (LightDiffuse[0] < 1.0f)
		{
			LightDiffuse[0] += 0.005f; 
			LightDiffuse[1] += 0.005f; 
			LightDiffuse[2] += 0.005f; 

			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}
	}
	else if (counter == 890)	// Scene is done
	{
		counter = -1;
		game_state++;
	}
	else if (counter > 700)		// Fade out scene
	{
		if (LightAmbient[0] > 0)
		{
			LightAmbient[0] -= 0.0025f;
			LightAmbient[1] -= 0.0025f;
			LightAmbient[2] -= 0.0025f;

			glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient );
			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}

		if (LightDiffuse[0] > 0)
		{
			LightDiffuse[0] -= 0.005f; 
			LightDiffuse[1] -= 0.005f; 
			LightDiffuse[2] -= 0.005f; 

			glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse );
		}
	}

	// Draw Flag

	glLoadIdentity();									// Reset The View
	glTranslatef(-1.05f,-1.25f, -5.0f);
	glRotatef(10.0f,1.0f,0.0f,0.0f);
	glBindTexture(GL_TEXTURE_2D, texture[7]);

	DrawFlag();

	// Draw Pole

	glTranslatef(-0.15f, 1.85f, 0);
	glRotatef(90.0f, 1.0f, 0, 0);
	glBindTexture(GL_TEXTURE_2D, texture[8]);

	gluCylinder(quadratic,0.15f,0.15f,6.0f,32,32);

	// Draw Pole Top

	gluSphere(quadratic,0.2f,32,32);

	glBindTexture(GL_TEXTURE_2D, texture[9]);

	// Draw Background

	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -34.0f);

	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-13.3f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 13.3f, -10.0f,  10.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 13.3f,  10.0f,  10.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-13.3f,  10.0f,  10.0f);
	glEnd();
}

int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

	switch(game_state)
	{
		case 0:
		{
			if (counter == 0)
			{
				PlaySound("data/sounds/fire1.wav",NULL,SND_ASYNC | SND_LOOP);
			}

			FadeInAndOut();
			
			glPrint(160, 135, 0, "Fire");

			break;
		}
		case 1:
		{
			if (counter==0)
			{
				change_view();
			}

			DrawFireScene();

			break;
		}
		case 2:
		{
			if (counter == 0)
			{
				change_view();
				PlaySound("data/sounds/water1.wav",NULL,SND_ASYNC);
			}

			FadeInAndOut();
			
			glPrint(150, 135, 0, "Water");

			break;
		}
		case 3:		// draw water
		{
			if (counter==0)
			{
				change_view();
			}

			DrawWaterScene();

			break;
		}
		case 4:		// wind intro
		{
			if (counter == 0)
			{
				change_view();
				glDisable(GL_LIGHTING);
				PlaySound("data/sounds/wind1.wav", NULL, SND_ASYNC | SND_LOOP);
			}

			FadeInAndOut();
			
			glPrint(150, 135, 0, "Wind");

			break;
		}
		case 5:
		{
			DrawWindScene();

			break;
		}
		case 6:
		{
			if (counter == 0)
			{
				glDisable(GL_LIGHTING);
				change_view();
			}

			FadeInAndOut();
			
			glPrint(110, 130, 0, "The End");

			break;
		}
		case 7:
		{
			FadeInAndOut();

			glPrint(80,90, 0, "GameDev.net");
			glPrint(70, 150, 2, "Four Elements III");
			glPrint(140, 180, 2, "Contest");

			break;
		}
		case 8:
		{
			FadeInAndOut();
			
			glPrint(60, 110, 0, "July 25, 2002");
			
			break;
		}
		case 9:
		{
			exit(0);
			
			break;
		}
	}

	counter++;

	UpdateFPS();

	return TRUE;										// Keep Going
}

GLvoid KillGLWindow(GLvoid)								// Properly Kill The Window
{
	if (fullscreen)										// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}

	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (!UnregisterClass("OpenGL",hInstance))			// Are We Able To Unregister Class
	{
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;									// Set hInstance To NULL
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style
	RECT		WindowRect;				// Grabs Rectangle Upper Left / Lower Right Values
	WindowRect.left=(long)0;			// Set Left Value To 0
	WindowRect.right=(long)width;		// Set Right Value To Requested Width
	WindowRect.top=(long)0;				// Set Top Value To 0
	WindowRect.bottom=(long)height;		// Set Bottom Value To Requested Height

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_SHELL);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= (LPCSTR)IDC_SHELL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP;										// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW;							// Windows Style
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		// Adjust Window To True Requested Size

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,							// Extended Style For The Window
								"OpenGL",							// Class Name
								title,								// Window Title
								dwStyle |							// Defined Window Style
								WS_CLIPSIBLINGS |					// Required Window Style
								WS_CLIPCHILDREN,					// Required Window Style
								0, 0,								// Window Position
								WindowRect.right-WindowRect.left,	// Calculate Window Width
								WindowRect.bottom-WindowRect.top,	// Calculate Window Height
								NULL,								// No Parent Window
								NULL,								// No Menu
								hInstance,							// Instance
								NULL)))								// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

//
// AboutProc()
// Handles messages sent to the About dialog
//
LRESULT CALLBACK AboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;		
			}
		break;
	}
	return FALSE;
}

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDM_EXIT:
					SendMessage(hWnd, WM_CLOSE, 0, 0);
					break;
				case IDM_ABOUT:
					DialogBox(NULL, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)AboutProc);
					break;
			}
			break;
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

	// Ask The User Which Screen Mode They Prefer
	//if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	//{
	//	fullscreen=FALSE;							// Windowed Mode
	//}

	// Create Our OpenGL Window
	if (!CreateGLWindow("",640,480,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
			// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !DrawGLScene()) || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
			{
				done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{
				SwapBuffers(hDC);					// Swap Buffers (Double Buffering)

				if (keys[VK_F1])						// Is F1 Being Pressed?
				{
					keys[VK_F1]=FALSE;					// If So Make Key FALSE
					KillGLWindow();						// Kill Our Current Window
					fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
					// Recreate Our OpenGL Window
					if (!CreateGLWindow("",640,480,16,fullscreen))
					{
						return 0;						// Quit If Window Was Not Created
					}
				}
			}
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
	return (msg.wParam);							// Exit The Program
}
